//**************************************************************************
//**
//** ai_egg.acs : poedDoom
//**$Revise Date: 11/21/07
//**$Author: kaiser
//**$Description: 'Egg (Hobbs form 1)' boss behavior
//**
//**************************************************************************

#define DEFAULTOFFSET 96
#define DEFAULTSPEED 2
int offset = DEFAULTOFFSET;
int speed = DEFAULTSPEED;
int eyeTid;
int eye[16] = 
{
	0,
	16<<8,
	32<<8,
	48<<8,
	64<<8,
	80<<8,
	96<<8,
	112<<8,
	128<<8,
	144<<8,
	160<<8,
	176<<8,
	192<<8,
	208<<8,
	224<<8,
	240<<8
};

bool EggDead = false;

script 913 (void)
{
	bossSubValue = GetActorProperty(masterTid, APROP_Health) / 100;
	delay(1);
	SetMusic("I_PROP");
	drawHealth = true;
}

script 914 (void)
{
	A_EggResetVars();
	tagEgg(_T(_I()));
	A_InitEgg();
	while(1)
	{
		if(EggDead == true)
			break;
		A_EggTicker();
		delay(1);
	}
}

script 915 (void)
{
	A_EyeSpitBolts("Egg_Flame2");
}

script 916 (void)
{
	str proj;
	int x, y, z;

	GiveInventory("QuestItem1", 1);

	x = GetActorX(masterTid);
	y = GetActorY(masterTid);
	z = GetActorZ(masterTid);

	if(random(0, 2) != 0)
		proj = "Egg_Bolt";
	else proj = "Egg_Flame2";

	while(offset < A_EggEye_Distance())
	{
		offset += (A_EggEye_Distance() >> 5);
		if(random(0, 4) != 0)
			A_EyeSpitBolts(proj);

		if(plyrZ > z)
			z += (16<<16);
		SetActorPosition(masterTid, x, y, z, false);

		delay(1);
	}
	while(offset > DEFAULTOFFSET)
	{
		offset -= (A_EggEye_Distance() >> 5);

		if(plyrZ > z)
			z += (16<<16);
		SetActorPosition(masterTid, x, y, z, false);

		delay(1);
	}

	offset = DEFAULTOFFSET;
	TakeInventory("QuestItem1", 1);
}

script 917 ENTER
{
	int colors;
	while(1)
	{
		if(drawHealth == true)
		{
			SetFont("CONFONT");
			SetHudSize(640, 480, 0);
			if(GetActorProperty(masterTid, APROP_Health) / bossSubValue < 40)
			colors = 6;
			else colors = 5;
			hudmessage(s:"Hobbs: ", i: GetActorProperty(masterTid, APROP_Health) / bossSubValue, s:"%"; HUDMSG_PLAIN,
				HUDBACK - 37, colors, 512.0, 32.0, 0.1);
		}
		delay(1);
	}
}

script 918 (void)
{
	int ticker;
	int i;

	for(i = 0; i < 16; i++)
		Thing_Remove(eyeTid+i);

	teleport_nofog(-123);

	EggDead = true;

	drawHealth = false;
	bossSubValue = 0;

	SetMusic("");
	ACS_Terminate(913, 0);
	ACS_Terminate(914, 0);
	ACS_Terminate(915, 0);
	ACS_Terminate(916, 0);
	ACS_Terminate(917, 0);

	while(ticker != 128)
	{
		if((random(0, 256) & 7) != 0)
			Spawn("WmullBoom", getActorX(masterTid) + (random(-256, 256) << 16),
			getActorY(masterTid) + (random(-256, 256) << 16), getActorZ(masterTid) + (random(0, 128) << 16), 0, 0);
		ticker++;
		delay(1);
	}
}

function void A_InitEgg(void)
{
	int i;

	eyeTid = GetAvailableSysTid();

	for(i = 0; i < 16; i++)
		A_SpawnInFront(GetActorX(masterTid), GetActorY(masterTid), GetActorZ(masterTid) + (128<<16),
			"Egg_Eye", eye[i], eyeTid+i, offset);
}

function int A_EggEye_Distance(void)
{
	int dist;
	dist = AproxDistance (GetActorX(masterTid) - plyrX, GetActorY(masterTid) - plyrY);
	dist = dist >> 16;

	return dist >> 2;
}

function void A_EyeSpitBolts(str projectile)
{
	int i, t;

	for(i = 0; i < 16; i++)
	{
		t = GetAvailableSysTid();
		Spawn(projectile, GetActorX(eyeTid+i), GetActorY(eyeTid+i), GetActorZ(eyeTid+i)-(32<<16), t, 0);
		ThrustThingZ(t, -128, 0, 0);
		Thing_ChangeTID(t, 0);
	}
}

function void A_EggTicker(void)
{
	int i;

	for(i = 0; i < 16; i++)
	{
		eye[i] = (eye[i] + (speed<<8)) & (255<<8);
		A_SetInFront(GetActorX(masterTid), GetActorY(masterTid), GetActorZ(masterTid) + (128<<16),
			eye[i], eyeTid+i, offset);
	}
}

function void A_EggResetVars(void)
{
	masterTid = 0;
	bossSubValue = 0;
	drawHealth = false;
}

function void tagEgg(int tid)
{
	masterTid = tid;
	Thing_ChangeTID (0, tid);
}
