//**************************************************************************
//**
//** gamfuncs.acs : poedDoom
//**$Revise Date: 11/21/06
//**$Author: kaiser
//**$Description: Common game functions that serve some kind of purpose
//**
//**************************************************************************

#library "gamfuncs"
#import "globals.c"
#import "indexes.c"
#import "boundaries.c"
#include "zcommon.acs"

//returns scientific notation. 10 ^X where X is depended on what number is entered. EX - 143 = 100, 15438 = 10000, etc.
function int GetNotation(int number) {
	bool foundNumber = false;
	int testNum = 1;
	int result;
	while(foundNumber == false) {
		if(number > testNum) {
			testNum *= 10;
		} else {
		result = testNum;
		foundNumber = true;
		}
	}
	return result / 10;
}

function int abs (int x) {
    if (x < 0)
        return -x;

    return x;
}

function int toChar (int dat) {
	if(dat >= 0xFF)
		return 0;

	if((dat & 0x7F) != dat)
		return abs((0xFF - dat) - 0x80);

	return dat;
}

function int toSByte (int dat) {
	if(dat >= 0xFF)
		return 0;

	if((dat & 0x7F) != dat)
		return -abs((0xFF - dat));

	return dat;
}

function int toByte (int dat) {
	return dat & 0xFF;
}

function int AproxDistance(int dx, int dy) {
  dx = abs(dx);
  dy = abs(dy);
  if (dx < dy)
    return dx+dy-(dx>>1);
  return dx+dy-(dy>>1);
}

function void A_SpawnInFront(int x, int y, int z, str type, int angle, int tid, int width) {
	int an2 = angle;
	int finecosine = fixedTable[an2 / 8 + 8192 / 4];
	int finesine = fixedTable[an2 / 8];
	Spawn(type, x + fixedmul(width*FRACUNITS*4, finecosine),
	y + fixedmul(width*FRACUNITS*4, finesine), z, tid, an2 / 256);
}

function void A_SetInFront(int x, int y, int z, int angle, int tid, int width) {
	int an2 = angle;
	int finecosine = fixedTable[an2 / 8 + 8192 / 4];
	int finesine = fixedTable[an2 / 8];
	SetActorPosition(tid, x + fixedmul(width*FRACUNITS*4, finecosine),
		y + fixedmul(width*FRACUNITS*4, finesine), z, false);
}

function void A_TossSomething(str type, int tid)
{
	Spawn(type, GetActorX(0), GetActorY(0), GetActorZ(0) + 32*FRACUNITS, 208,
	random(0, 360));
	ThrustThing(GetActorAngle(208) >> 8, random(-8, 8), 0, 208);
	ThrustThingZ(208, random(16, 32), 0, 0);
	Thing_ChangeTID (208, tid);
}

function int A_ReachPlayerLen(int x, int y) {
	int distance;
	distance = AproxDistance (PlyrX - x, PlyrY - y);
	distance = distance >> 16;

	return distance;
}

function void LockPlayer(void) {
	SetPlayerProperty(1, 1, 4);
	SetPlayerProperty(1, 1, 1);
}

function void UnLockPlayer(void) {
	SetPlayerProperty(1, 0, 4);
	SetPlayerProperty(1, 0, 1);
}

function void ResetLinedef(int arg1) {
	setlinetexture(arg1, SIDE_FRONT, TEXTURE_MIDDLE, "-");
	setlinetexture(arg1, SIDE_BACK, TEXTURE_MIDDLE, "-");
	setlineblocking(arg1, BLOCK_NOTHING);
}

function void SpawnMobjMissile(str type, str sound, bool thrustZ,
							   int offset, int distance, int height, int speed) {
	int heightShift = 0;
	int pitchShift = 0;
	int tTid = GetAvailableSysTid();
	int x = GetActorX(0);
	int y = GetActorY(0);
	int z = GetActorZ(0);
	int p = GetActorPitch(0);
	int a = GetActorAngle(0);

	if(TryCollide(x, y, a))
		return;

	if((z + (64 << 16)) >= GetActorCeilingZ(0))
		heightShift = 16 << 16;

	pitchShift = (p << 8);

	Spawn(type, x + distance * cos (a + offset), y + distance * sin (a + offset),
			(z + (height << 16) - heightShift) - pitchShift, tTid, a >> 8);

	if(thrustZ)
		ThrustThingZ(tTid, (pitchShift*-1) >> 14, 0, 0);

	ThingSound(tTid, sound, 127);
	ThrustThing(GetActorAngle(tTid) >> 8, speed, 0, tTid);
}
