#include <stdio.h>
#include <io.h>
#include <string.h>
#include <stdlib.h>

typedef unsigned char byte;
typedef unsigned int uint;
typedef unsigned short ushort;

/*Wad structs*/

typedef struct
{
	char		identification[4];
	int			numlumps;
	int			infotableofs;
} wadinfo_t;

typedef struct
{
	int			filepos;
	int			size;
	char		name[8];
} lumpinfo_t;

enum { ML_LABEL, ML_THINGS, ML_LINEDEFS, ML_SIDEDEFS, ML_VERTEXES, ML_SEGS,
ML_SSECTORS, ML_NODES, ML_SECTORS , ML_REJECT, ML_BLOCKMAP, ML_BEHAVIOR, NUMOFLUMPS };

typedef struct {
	short v1, v2;
	short flags;
	byte args[6];
	short front;
	short back;
} doomLine_t;

typedef struct {
	short x, y;
	char upper[8];
	char lower[8];
	char middle[8];
	short sector;
} doomSide_t;

typedef struct {
	signed short x, y;
} doomVertex_t;

int main(int argc, char** argv) {
	byte* buffer;
	char WadFile[2][256];
	FILE* f;
	FILE* acs;
	int i, size, lmpSize[NUMOFLUMPS], k, vertex1, vertex2, count = 0;
	doomLine_t* line;
	doomSide_t* side;
	doomVertex_t* vertex;
	wadinfo_t wad;
	lumpinfo_t lump[NUMOFLUMPS];

	if(!argv[1])
		return 0;

	for(i = 0; i < 100; i++) {
		sprintf(WadFile[1], "%sMAP%02d.wad", argv[1], i);
		f = fopen(WadFile[1], "rb");

		if(!f)
			break;
		else {
			printf("Reading %s\n", WadFile[1]);
			fseek(f, 0, SEEK_END);
			size = ftell(f);
			fseek(f, 0, SEEK_SET);

			buffer = (byte*)malloc(size);
			fread(buffer, size, 1, f);
			fclose(f);

			sprintf(WadFile[0], "%sBoundMap%02d.c", argv[1], i);
			acs = fopen(WadFile[0], "w");

			fprintf(acs, "/*Generated from BNDMAKE.EXE*/\n\n");
			for(k = 0; k < 33; k++)
				fprintf(acs, " ");

			fprintf(acs, "int BoundMap%02d[BOUNDARIESCOUNT%02d][5] = {\n", i, i);
			fprintf(acs, "/*************************Map%02d***********************/\n", i);

			memset(&wad, 0, sizeof(wadinfo_t));
			memset(&lump, 0, sizeof(lumpinfo_t) * NUMOFLUMPS);

			memcpy(&wad, buffer, sizeof(wadinfo_t));
			memcpy(&lump, buffer + wad.infotableofs, sizeof(lumpinfo_t) * NUMOFLUMPS);

			/*Linedefs*/
			lmpSize[ML_LINEDEFS] = lump[ML_LINEDEFS].size / sizeof(doomLine_t);
			printf("Number of linedefs: %i\n", lmpSize[ML_LINEDEFS]);
			line = (doomLine_t*)malloc(sizeof(doomLine_t) * lmpSize[ML_LINEDEFS]);
			memcpy(line, buffer + lump[ML_LINEDEFS].filepos, lump[ML_LINEDEFS].size);

			/*Sidedefs*/
			lmpSize[ML_SIDEDEFS] = lump[ML_SIDEDEFS].size / sizeof(doomSide_t);
			printf("Number of sidedefs: %i\n", lmpSize[ML_SIDEDEFS]);
			side = (doomSide_t*)malloc(sizeof(doomSide_t) * lmpSize[ML_SIDEDEFS]);
			memcpy(side, buffer + lump[ML_SIDEDEFS].filepos, lump[ML_SIDEDEFS].size);

			/*Vertex*/
			lmpSize[ML_VERTEXES] = lump[ML_VERTEXES].size / sizeof(doomVertex_t);
			printf("Number of vertexes: %i\n", lmpSize[ML_VERTEXES]);
			vertex = (doomVertex_t*)malloc(sizeof(doomVertex_t) * lmpSize[ML_VERTEXES]);
			memcpy(vertex, buffer + lump[ML_VERTEXES].filepos, lump[ML_VERTEXES].size);

			count = 0;
			for(k = 0; k < lmpSize[ML_LINEDEFS]; k++) {
				if(line[k].back == -1 && !(line[k].flags & 0x4000)) {
					count++;
					fprintf(acs, "\t{ ");
					fprintf(acs, "%i, ", k+1);
					printf("Reading %i vertexes: x%i, y%i and x%i, y%i\n", k, vertex[line[k].v1].x,
						vertex[line[k].v2].x, vertex[line[k].v1].y, vertex[line[k].v2].y);
					vertex1 = (vertex[line[k].v1].x | (vertex[line[k].v2].x << 16));
					vertex2 = (vertex[line[k].v1].y | (vertex[line[k].v2].y << 16));
					fprintf(acs, "%i, %i, %i, %i },\n", vertex[line[k].v1].x, vertex[line[k].v2].x,
						vertex[line[k].v1].y, vertex[line[k].v2].y);
				}
			}
			if(count >= 4096) {
				printf("Error: Map%02d has too many lines (%i > 4096)\n", i, count);
				return 0;
			}
			fprintf(acs, "\t\t/******Boundaries: %i*********/\n\n", count);
			fseek(acs, 34, SEEK_SET);
			fprintf(acs, "#define BOUNDARIESCOUNT%02d\t\t%i\n", i, count);
			fseek(acs, 0, SEEK_END);

			free(line);
			free(side);
			free(vertex);

			fprintf(acs, "};\n\n");
			fclose(acs);
		}
		if(buffer != NULL)
			free(buffer);
	}

	return 1;
}