//-------------------------------------------------------------------------
// STRMAKE - String to ascii char converter
// Turns a simple string into a per integer block array for Zdoom ACS scripting
// Primary used for Community is Falling 3 (Kaiser_21.wad)
//
// (c) 11/15/2006 Samuel 'Kaiser' Villarreal
// Wow.. this tool is a year old already.. to celebrate, strmake will now have
// string encoding support, making it harder to figure out how the strings work
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <io.h>
#include <string.h>
#include <stdlib.h>
#include "encoders.h"

/*__________________________________________________________________
Global Defines
____________________________________________________________________*/

nameTree_t nameTree[MAXSLOTS];
strTree_t strTree;
int match = 0;
int listPos = 0;

/*__________________________________________________________________
K_SetupStrTree
____________________________________________________________________*/

void K_SetupStrTree(void) {
	int i, k, len, n = 0;
	byte node = 0;

	for(i = 0; i < slot; i++) {
		len = strlen(Text[i]);
		for(k = 0; k < len; k++) {
			if(!strTree.symbol[Text[i][k]]) {
				strTree.symbol[Text[i][k]] = Text[i][k];
				strTree.count[Text[i][k]]++;
			}
			else
				strTree.count[Text[i][k]]++;
		}
	}

	strTree.max = i;

	for(i = 0; i < slot; i++)
		K_InsertNameIndex(i);

	K_SortStrTree();

	for(i = 0; i < 256; i++, strTree.dirSize++) {
		if(!strTree.count[i]) break;
		if(i % MAXSYMBOLS == 0) {
			n = 0;
		}
		strTree.index[i] = (1 << (n+1));
		strTree.index[i] |= node;

		printf("%c: %i %i : %i\n", strTree.symbol[i], strTree.count[i],
			i, strTree.index[i]);
		node ^= 1;
		if(node == 0) n++;
	}

	K_MakeStrTree();
}

/*__________________________________________________________________
K_SortStrTree
____________________________________________________________________*/

void K_SortStrTree(void) {
	int i, k, cpyCount = 0;
	char cpyChar = 0;

	for(i = 0; i < 256; i++) {
		for(k = 0; k < 256; k++) {
			if(strTree.count[i] >= strTree.count[k]) {
				cpyCount = strTree.count[i];
				cpyChar = strTree.symbol[i];
				strTree.count[i] = strTree.count[k];
				strTree.symbol[i] = strTree.symbol[k];
				strTree.count[k] = cpyCount;
				strTree.symbol[k] = cpyChar;
			}
		}
	}
}

/*__________________________________________________________________
K_GetBinCount
____________________________________________________________________*/

int K_GetBinCount(int num) {
	int i, bit = 1;

	for(i = 0; i < 32; i++) {
		if((num & (bit - 1)) == num) return i;
		else bit = (bit << 1);
	}

	printf("Error: couldn't get bincount from %i\n", num);
	exit(-1);
	return -1;
}

/*__________________________________________________________________
K_LookUpSymbol
____________________________________________________________________*/

byte K_LookUpSymbol(byte symbol) {
	int i;

	for(i = 0; i < 256; i++)
		if(strTree.symbol[i] == symbol)
			return i;
	return i;
}

/*__________________________________________________________________
K_WriteStrTree
____________________________________________________________________*/

void K_WriteStrTree(void) {
	int i, k, b;
	char header[4];
	byte getshiftbytes = 0;
	short shiftbytes;
	FILE* f = fopen("temp.lmp", "wb");

	header[0] = 'D';
	header[1] = 'I';
	header[2] = 'R';

	fwrite(&header, 3, 1, f);
	fwrite(&strTree.dirSize, 1, 1, f);

	for(i = 0; strTree.count[i] != 0; i++)
		fwrite(&strTree.symbol[i], 1, 1, f);

	header[0] = 'X';
	header[1] = 'S';
	header[2] = 'T';
	header[3] = 'R';

	fwrite(&header, 4, 1, f);
	fwrite(&strTree.max, 2, 1, f);

	for(k = 0; k < strTree.max; k++)
		fwrite(&strTree.strSize[k], 1, 1, f);

	for(k = 0; k < strTree.max; k++)
		for(i = 0; i < strTree.strSize[k]; i++) {
			for(b = 0; b < 8; b++) {
				if(strTree.codeString[k][i+b] == 0x00)
				getshiftbytes = getshiftbytes | (1 << b);
			}

			fwrite(&(byte)getshiftbytes, 1, 1, f);

			for(b = 0; b < 8; b++, i++) {
				if(strTree.codeString[k][i] != 0x00)
					fwrite(&strTree.codeString[k][i], 1, 1, f);
			}

			i--;
			getshiftbytes = 0;
			shiftbytes = 0;
		}

	fclose(f);
}

/*__________________________________________________________________
K_MakeStrTree
____________________________________________________________________*/

void K_MakeStrTree(void) {
	int i = 0, k = 0, len, idPos = 0, str, b;
	byte lookup, idByte = 0;
	for(str = 0; str < strTree.max; str++) {

		len = strlen(Text[str]);

		idPos = 0;
		idByte = 0;

		for(k = 0; k < len; k++) {

			lookup = K_LookUpSymbol(Text[str][k]);

			for(i = 0; i < K_GetBinCount(strTree.index[lookup]); i++) {

				if((strTree.index[lookup] >> i) & 1) {
					if(i > 0 && lookup >= MAXSYMBOLS) {
						for(b = 0; b < MAXBITS; b++) {
							idByte = (idByte + 1) & 7;
							if(!idByte) idPos++;
						}
					}
					strTree.codeString[str][idPos] |= (1 << idByte);
				}
				idByte = (idByte + 1) & 7;

				if(!idByte) idPos++;
			}
		}
		strTree.strSize[str] = idPos + idByte;
	}

	K_WriteStrTree();
}

/*__________________________________________________________________
K_OpenStrTree
____________________________________________________________________*/

void K_OpenStrTree(char* bin) {
	byte* buffer, *tmp;
	int size, i, pos, k, index;
	FILE* f = fopen(bin, "rb");
	strTree_t tree;
	short getidbyte = 0;
	byte lookupid = 0, strPos = 0;
	char string[MAXSLOTS][256];
	byte codeData[256];
	byte bits = 0;

	if(!f)
		return;

	fseek(f, 0, SEEK_END);
	size = ftell(f);
	fseek(f, 0, SEEK_SET);

	tmp = buffer = (byte*)malloc(size);

	fread(buffer, size, 1, f);

	if(*buffer++ == 'D' && *buffer++ == 'I' && *buffer++ == 'R')
		tree.dirSize = *buffer++;
	else {
		printf("Error: Header not found\n");
		exit(-1);
	}

	printf("Size: %i\n", tree.dirSize);

	for(i = 0; i < tree.dirSize; i++) {
		tree.symbol[i] = *buffer++;
		printf("Loaded symbol %c : %i\n", tree.symbol[i], i);
	}


	if(*buffer++ == 'X' && *buffer++ == 'S' && *buffer++ == 'T' && *buffer++ == 'R') {
		tree.max = *buffer++;
		tree.max |= ((*buffer++) << 8);
	}
	else {
		printf("Error: String Header not found\n");
		exit(-1);
	}


	for(i = 0; i < tree.max; i++)
		tree.strSize[i] = *buffer++;

	printf("Strings found: %i\n", tree.max);

	for(index = 0; index < tree.max; index++) {
		getidbyte = 0;
		lookupid = 0;
		strPos = 0;

		memset(&string, 0, 256);
		memset(&codeData, 0, 256);

		k = 0;
		bits = 0;

		do {
			if(!bits)
				lookupid = buffer[pos++];

			bits = (bits + 1) & 7;

			if(lookupid & 1)
				codeData[k++] = 0x00;
			else
				codeData[k++] = buffer[pos++];

			lookupid = lookupid >> 1;

		} while(k < tree.strSize[index]);

		lookupid = 0;

		for(i = 0; i < tree.strSize[index]; i++) {
			for(k = 0; k < 8; k++) {
				if(getidbyte == 0) {
					lookupid = ((codeData[i] >> k) & 1);
					getidbyte++;
				}
				else {
					if(((codeData[i] >> k) & 1) == 0)
						getidbyte++;
					else {
						string[index][strPos++] = tree.symbol[((getidbyte-1) << 1) + lookupid];
						getidbyte = 0;
					}
				}
			}
		}

		printf("%s\n\n", string[index]);
	}

	free(tmp);

	fclose(f);
}

/*__________________________________________________________________
K_BreakUpNames
____________________________________________________________________*/

void K_BreakUpNames(void) {
	int i, len, j, pos, slotx = slot-1;

	for(i = 0; i < slot; i++) {
		len = strlen(Name[i]);
		strncpy(nameTree[listPos++].cpyName, Name[i], 32);

		j = 0;
		for(pos = 0; pos < len; pos++) {
			if(Name[i][pos] == 0x20) {
				slotx++;
				j = 0;
				while(Name[i][pos+1 + j] != 0x20 && Name[i][pos+1 + j] != 0) {
					Name[slotx][j++] = Name[i][pos+1 + j];
				}
				Name[i][pos] = 0;
			}
		}
	}
	match = slotx;
}

/*__________________________________________________________________
K_SortNameTree
____________________________________________________________________*/

void K_SortNameTree(void) {
	int i, k, cpyCount = 0;
	char cpyChar[32];

	for(i = 0; i < match; i++) {
		for(k = 0; k < match; k++) {
			if(nameTree[i].count >= nameTree[k].count) {
				cpyCount = nameTree[i].count;
				strncpy(cpyChar, nameTree[i].nameTree, 32);
				nameTree[i].count = nameTree[k].count;
				strncpy(nameTree[i].nameTree, nameTree[k].nameTree, 32);
				nameTree[k].count = cpyCount;
				strncpy(nameTree[k].nameTree, cpyChar, 32);
			}
		}
	}
}

/*__________________________________________________________________
K_CreateNameDir
____________________________________________________________________*/

void K_CreateNameDir(void) {
	int i, k, len, n = 0, sort = 0;
	byte node = 2;
	byte singleIndex = 0;
	int names;

	K_BreakUpNames();

	names = match;
	match = 0;

	for(i = 0; i < names; i++) {
		for(k = 0; k < match; k++) {
			if(!strcmp(nameTree[k].nameTree, Name[i])) {
				nameTree[k].count++;
				break;
			}
		}
		if(k >= match) {
			len = strlen(Name[i]);
			strncpy(nameTree[match].nameTree, Name[i], len);
			match++;
		}
	}
	K_SortNameTree();
}

/*__________________________________________________________________
K_FindString
____________________________________________________________________*/

int K_FindString(char* search, char* text, bool_t checkName) {
	int nLen = strlen(search);
	int sLen = strlen(text);
	int i = 0, k = 0;

	if(checkName == true) {
		if(search[0] == '#')
			return -1;

		if(nLen <= 3) return -1;
	}

	for(i = 0; i < sLen; i++) {
		if(search[0] == text[i]) {
			for(k = 0; k < nLen; k++) {
				if(search[k] != text[i + k]) break;
			}
			if(k >= nLen)
				return i;
		}
	}

	return -1;
}

/*__________________________________________________________________
K_InsertNameIndex
____________________________________________________________________*/

void K_InsertNameIndex(int index) {
	byte cpyStr[256];
	int i = 0, pos, k = 0;
	int sLen = strlen(Text[index])-1;
	int nLen;

	memset(&cpyStr, 0, 256);

	for(i = 0; i < match; i++) {
		pos = K_FindString(nameTree[i].nameTree, Text[index], true);
		if(pos != -1) {
			for(k = 0; k < pos; k++) {
				cpyStr[k] = Text[index][k];
			}

			cpyStr[pos] = strTree.symbol['$'] = '$';
			cpyStr[pos+1] = strTree.symbol[i] = i;
			strTree.count['$']++;
			strTree.count[i]++;

			nLen = strlen(nameTree[i].nameTree)-1;

			for(k = pos+2; k < sLen+2; k++) {
				cpyStr[k] = Text[index][nLen-1 + k];
			}
			strncpy(Text[index], cpyStr, 256);
		}
	}
}

/*__________________________________________________________________
K_AssembleNames
____________________________________________________________________*/

void K_AssembleNames(void) {
	int i, k, pos = 0;
	byte getidbyte = 0;

	for(i = 0; i < slot; i++) {
		getidbyte = 0;
		memset(&nameTree[i].treeIndex, -1, sizeof(int)*8);
		for(k = 0; k < match; k++) {
			pos = K_FindString(nameTree[k].nameTree, nameTree[i].cpyName, false);
			if(pos != -1) {
				nameTree[i].treeIndex[getidbyte] = pos;
				nameTree[i].treeIndex[getidbyte] |= (k << 16);
				getidbyte++;
			}
		}
	}
}

/*__________________________________________________________________
K_TestNames
____________________________________________________________________*/

void K_TestNames(void) {
	int i, k;

	for(i = 0; i < slot; i++) {
		for(k = 0; k < 8; k++) {
			if(nameTree[i].treeIndex[k] == -1)
				break;
			else {
				printf("%s:%i ", nameTree[nameTree[i].treeIndex[k] >> 16].nameTree,
					nameTree[i].treeIndex[k] & 0xffff);
			}
		}
		printf("\n");
	}
}
