//-------------------------------------------------------------------------
// STRMAKE - String to ascii char converter
// Turns a simple string into a per integer block array for Zdoom ACS scripting
// Primary used for Community is Falling 3 (Kaiser_21.wad)
//
// (c) 11/15/2005 Samuel 'Kaiser' Villarreal
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <io.h>
#include <string.h>
#include <stdlib.h>
#include "encoders.h"

/*__________________________________________________________________
Global Defines
____________________________________________________________________*/

char * buffer;
char * endBuffer;
int slot = 0;
//on screen text
char Name[MAXSLOTS][32];	//character name string
char Text[MAXSLOTS][256];	//dialog text string

/*__________________________________________________________________
K_OpenFile
____________________________________________________________________*/

void K_OpenFile(char* input) {
	long lSize;
	FILE* f;
	f = fopen ( input , "rb" );
	if (f==NULL) exit (1);
	fseek (f , 0 , SEEK_END);
	lSize = ftell (f);
	rewind (f);
	buffer = (char*) malloc (lSize + 1);
	if (buffer == NULL) exit (2);
	endBuffer = buffer + lSize;
	fread (buffer,1,lSize,f);
	fclose (f);
}

/*__________________________________________________________________
K_MakeACSArray
____________________________________________________________________*/

void K_MakeACSArray(char* input) {
	FILE* f;
	FILE* strBuf;
	int strBufSize;
	byte* strBufDat;
	int i, k;
	int len, nameData;
	byte nameEx;
	char* newFile = input+strlen(input)-1;
	while(newFile != input) {
		if(*newFile == 0x5c)
		{
			newFile++;
			*newFile = '\0';
			break;
		}
		newFile--;
	}
	strcat(input, "acsStrings.acs");
	f = fopen(input, "w");
	if (f==NULL) {
		printf("Read Error..\n");
		exit (1);
	}
	fprintf(f, "//Text file generated by STRMAKE.exe\n\n");
	fprintf(f, "//Names\n");
	for(i = 0; i < match; i++) {
		//write name
		len = strlen(nameTree[i].nameTree);
		nameEx = len % 4;

		len += nameEx;

		if(i == 0) fprintf(f, "int nameTree[%i][8] = {\n", match);


		fprintf(f, "/*%i: %s*/\n\t\t", i, nameTree[i].nameTree);

		for(k = 0; k < 28; k+=4) {
			if(k == 0) fprintf(f, "	{ ");
			nameData = 
			((nameTree[i].nameTree[k])|((nameTree[i].nameTree[k+1])<<8)|
			((nameTree[i].nameTree[k+2])<<16)|((nameTree[i].nameTree[k+3])<<24));
			(k >=24) ? fprintf(f, "%i }", nameData) :
				fprintf(f, "%i,", nameData);
		}
		i >= match-1 ? fprintf(f, "\n};\n") : fprintf(f, ",\n");
	}

	K_AssembleNames();

	fprintf(f, "\n\n//Name Assembly List\n");
	fprintf(f, "int nameList[%i][8] = {\n\n", slot);

	for(i = 0; i < slot; i++) {
		fprintf(f, "/*%i: %s*/\n", i, nameTree[i].cpyName);
		fprintf(f, "\t\t\t{ ");
		for(k = 0; k < 8; k++)
			k==8-1?fprintf(f, "%i ", nameTree[i].treeIndex[k]):
				fprintf(f, "%i,", nameTree[i].treeIndex[k]);

		i == slot-1?fprintf(f, "}\n"):fprintf(f, "},\n");
	}

	fprintf(f, "};\n");

	fprintf(f, "\n");

	fprintf(f, "\n\n//Strings\n\n");
	for(i = 0; i < slot; i++) {
		fprintf(f, "/*%i: %s*/\n", i, Text[i]);
	}
	strBuf = fopen("temp.lmp", "rb");
	fseek(strBuf, 0, SEEK_END);
	strBufSize = ftell(strBuf);
	fseek(strBuf, 0, SEEK_SET);

	strBufDat = (byte*)malloc(strBufSize);
	fread(strBufDat, 1, strBufSize, strBuf);
	fclose(strBuf);

	k = 0;
	i = 0;

	fprintf(f, "\n\n");
	fprintf(f, "int strDirSize = %i;\n", strBufSize);

	while(strBufSize % 4)
		strBufSize++;

	fprintf(f, "int strDat[%i] = {\n\n", strBufSize / 4);

	for(i = 0; i < strBufSize; i+=4) {

		k = (strBufDat[i]|(strBufDat[1+i]<<8)|(strBufDat[2+i]<<16)
			|(strBufDat[3+i]<<24));

		fprintf(f, "%i", k);
		(i+4 < strBufSize)?fprintf(f, ", "):fprintf(f, " ");

		if(i >= strBufSize) break;
		if(!(i % 32)) fprintf(f, "\n");
	}

	fprintf(f, "};\n");

	free(strBufDat);

	newFile = 0;
	fclose(f);
}

/*__________________________________________________________________
main
____________________________________________________________________*/

int main(int argc, char**argv)
{
	int pos = 0;
	int i;
	bool_t foundToken = false;

#ifdef DEBUG
	if(argv[2] && argv[3]) {
		K_OpenStrTree(argv[2]);
		return 1;
	}
#endif /*DEBUG*/

	if(argc == 1 || argc > 2) {
		printf("STRMAKE by Samuel 'Kaiser' Villarreal - Usage: STRMAKE.exe 'location of file'\n");
		return 1;
	}
	K_OpenFile(argv[1]);
	for(i = 0; i < 256; i++) {
		memset(Name, 0, sizeof(Name[i]));
		memset(Text, 0, sizeof(Text[i]));
	}
	printf("============================\nSTRMAKE by Samuel 'Kaiser' Villarreal\n============================\nReading file...\n");
	while(buffer < endBuffer) {
		if(*buffer <= 32) {
			buffer++;
		}
		else if(*buffer == '$') {
			foundToken = true;
			//bypass '$'
			buffer++;
			while(*buffer != '=') {
				if(pos > sizeof(Name) / 256) {
					printf("String is too long..\n");
					exit(1);
				}
				if(*buffer < 32) {
					printf("String Error..\n");
					exit(1);
				}
				Name[slot][pos] = *buffer;
				pos++;
				buffer++;
			}
			//bypass '='
			buffer++;
			pos = 0;
			while(*buffer >= 32) {
				if(pos > sizeof(Text) / 256) {
					printf("String is too long..\n");
					exit(1);
				}
				Text[slot][pos] = *buffer;
				pos++;
				buffer++;
			}
			pos = 0;
			slot++;
			foundToken = false;
		}
		else if(*buffer == '\n') {
			buffer++;
		}
		else {
			printf("Unknown token found..\n");
			exit(1);
		}
	}
	buffer = 0;
	free (buffer);
	K_CreateNameDir();
	K_SetupStrTree();
	K_MakeACSArray(argv[1]);

#ifdef DEBUG
	K_TestNames();
#endif /*DEBUG*/

	printf("Done..%i array slots left\n", MAXSLOTS - slot);
	return 0;
}